__version__ = '0.6.4'

try:
    from mando.core import Program
except ImportError as e:  # pragma: no cover
    # unfortunately the only workaround for Python2.6, argparse and setup.py
    e.version = __version__
    raise e

main = Program()
command = main.command
arg = main.arg
parse = main.parse
execute = main.execute



# -*- coding: utf-8 -*-
"""
    sphinx.ext.napoleon.docstring
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


    Classes for docstring parsing and formatting.


    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""

import collections
import inspect
import re

from six import string_types, u
from six.moves import range

from mando.napoleon.iterators import modify_iter
from mando.napoleon.pycompat import UnicodeMixin


_directive_regex = re.compile(r'\.\. \S+::')
_google_section_regex = re.compile(r'^(\s|\w)+:\s*$')
_google_typed_arg_regex = re.compile(r'\s*(.+?)\s*\(\s*(.*[^\s]+)\s*\)')
_numpy_section_regex = re.compile(r'^[=\-`:\'"~^_*+#<>]{2,}\s*$')
_single_colon_regex = re.compile(r'(?<!:):(?!:)')
_xref_regex = re.compile(r'(:(?:[a-zA-Z0-9]+[\-_+:.])*[a-zA-Z0-9]+:`.+?`)')
_bullet_list_regex = re.compile(r'^(\*|\+|\-)(\s+\S|\s*$)')
_enumerated_list_regex = re.compile(
    r'^(?P<paren>\()?'
    r'(\d+|#|[ivxlcdm]+|[IVXLCDM]+|[a-zA-Z])'
    r'(?(paren)\)|\.)(\s+\S|\s*$)')


class GoogleDocstring(UnicodeMixin):
    """Convert Google style docstrings to reStructuredText.

    Parameters
    ----------
    docstring : :obj:`str` or :obj:`list` of :obj:`str`
        The docstring to parse, given either as a string or split into
        individual lines.
    config: :obj:`sphinx.ext.napoleon.Config` or :obj:`sphinx.config.Config`
        The configuration settings to use. If not given, defaults to the
        config object on `app`; or if `app` is not given defaults to the
        a new :class:`sphinx.ext.napoleon.Config` object.


    Other Parameters
    ----------------
    app : :class:`sphinx.application.Sphinx`, optional
        Application object representing the Sphinx process.
    what : :obj:`str`, optional
        A string specifying the type of the object to which the docstring
        belongs. Valid values: "module", "class", "exception", "function",
        "method", "attribute".
    name : :obj:`str`, optional
        The fully qualified name of the object.
    obj : module, class, exception, function, method, or attribute
        The object to which the docstring belongs.
    options : :class:`sphinx.ext.autodoc.Options`, optional
        The options given to the directive: an object with attributes
        inherited_members, undoc_members, show_inheritance and noindex that
        are True if the flag option of same name was given to the auto
        directive.


    Example
    -------
    >>> from sphinx.ext.napoleon import Config
    >>> config = Config(napoleon_use_param=True, napoleon_use_rtype=True)
    >>> docstring = '''One line summary.
    ...
    ... Extended description.
    ...
    ... Args:
    ...   arg1(int): Description of `arg1`
    ...   arg2(str): Description of `arg2`
    ... Returns:
    ...   str: Description of return value.
    ... '''
    >>> print(GoogleDocstring(docstring, config))
    One line summary.
    <BLANKLINE>
    Extended description.
    <BLANKLINE>
    :param arg1: Description of `arg1`
    :type arg1: int
    :param arg2: Description of `arg2`
    :type arg2: str
    <BLANKLINE>
    :returns: Description of return value.
    :rtype: str
    <BLANKLINE>

    """
    def __init__(self, docstring, config=None, app=None, what='', name='',
                 obj=None, options=None):
        # type: (Union[unicode, List[unicode]], SphinxConfig, Sphinx, unicode, unicode, Any, Any) -> None  # NOQA
        self._config = config
        self._app = app

        if not self._config:
            from sphinx.ext.napoleon import Config
            self._config = self._app and self._app.config or Config()  # type: ignore

        if not what:
            if inspect.isclass(obj):
                what = 'class'
            elif inspect.ismodule(obj):
                what = 'module'
            elif isinstance(obj, collections.Callable):  # type: ignore
                what = 'function'
            else:
                what = 'object'

        self._what = what
        self._name = name
        self._obj = obj
        self._opt = options
        if isinstance(docstring, string_types):
            docstring = docstring.splitlines()
        self._lines = docstring
        self._line_iter = modify_iter(docstring, modifier=lambda s: s.rstrip())
        self._parsed_lines = []  # type: List[unicode]
        self._is_in_section = False
        self._section_indent = 0
        if not hasattr(self, '_directive_sections'):
            self._directive_sections = []  # type: List[unicode]
        if not hasattr(self, '_sections'):
            self._sections = {
                'args': self._parse_parameters_section,
                'arguments': self._parse_parameters_section,
                'attributes': self._parse_attributes_section,
                'example': self._parse_examples_section,
                'examples': self._parse_examples_section,
                'keyword args': self._parse_keyword_arguments_section,
                'keyword arguments': self._parse_keyword_arguments_section,
                'methods': self._parse_methods_section,
                'note': self._parse_note_section,
                'notes': self._parse_notes_section,
                'other parameters': self._parse_other_parameters_section,
                'parameters': self._parse_parameters_section,
                'return': self._parse_returns_section,
                'returns': self._parse_returns_section,
                'raises': self._parse_raises_section,
                'references': self._parse_references_section,
                'see also': self._parse_see_also_section,
                'todo': self._parse_todo_section,
                'warning': self._parse_warning_section,
                'warnings': self._parse_warning_section,
                'warns': self._parse_warns_section,
                'yield': self._parse_yields_section,
                'yields': self._parse_yields_section,
            }  # type: Dict[unicode, Callable]
        self._parse()

    def __unicode__(self):
        # type: () -> unicode
        """Return the parsed docstring in reStructuredText format.

        Returns
        -------
        unicode
            Unicode version of the docstring.

        """
        return u('\n').join(self.lines())

    def lines(self):
        # type: () -> List[unicode]
        """Return the parsed lines of the docstring in reStructuredText format.

        Returns
        -------
        list(str)
            The lines of the docstring in a list.

        """
        return self._parsed_lines

    def _consume_indented_block(self, indent=1):
        # type: (int) -> List[unicode]
        lines = []
        line = self._line_iter.peek()
        while(not self._is_section_break() and
              (not line or self._is_indented(line, indent))):
            lines.append(next(self._line_iter))  # type: ignore
            line = self._line_iter.peek()
        return lines

    def _consume_contiguous(self):
        # type: () -> List[unicode]
        lines = []
        while (self._line_iter.has_next() and
               self._line_iter.peek() and
               not self._is_section_header()):
            lines.append(next(self._line_iter))  # type: ignore
        return lines

    def _consume_empty(self):
        # type: () -> List[unicode]
        lines = []
        line = self._line_iter.peek()
        while self._line_iter.has_next() and not line:
            lines.append(next(self._line_iter))  # type: ignore
            line = self._line_iter.peek()
        return lines

    def _consume_field(self, parse_type=True, prefer_type=False):
        # type: (bool, bool) -> Tuple[unicode, unicode, List[unicode]]
        line = next(self._line_iter)  # type: ignore

        before, colon, after = self._partition_field_on_colon(line)
        _name, _type, _desc = before, '', after  # type: unicode, unicode, unicode

        if parse_type:
            match = _google_typed_arg_regex.match(before)  # type: ignore
            if match:
                _name = match.group(1)
                _type = match.group(2)

        _name = self._escape_args_and_kwargs(_name)

        if prefer_type and not _type:
            _type, _name = _name, _type
        indent = self._get_indent(line) + 1
        _descs = [_desc] + self._dedent(self._consume_indented_block(indent))
        _descs = self.__class__(_descs, self._config).lines()
        return _name, _type, _descs

    def _consume_fields(self, parse_type=True, prefer_type=False):
        # type: (bool, bool) -> List[Tuple[unicode, unicode, List[unicode]]]
        self._consume_empty()
        fields = []
        while not self._is_section_break():
            _name, _type, _desc = self._consume_field(parse_type, prefer_type)
            if _name or _type or _desc:
                fields.append((_name, _type, _desc,))
        return fields

    def _consume_inline_attribute(self):
        # type: () -> Tuple[unicode, List[unicode]]
        line = next(self._line_iter)  # type: ignore
        _type, colon, _desc = self._partition_field_on_colon(line)
        if not colon:
            _type, _desc = _desc, _type
        _descs = [_desc] + self._dedent(self._consume_to_end())
        _descs = self.__class__(_descs, self._config).lines()
        return _type, _descs

    def _consume_returns_section(self):
        # type: () -> List[Tuple[unicode, unicode, List[unicode]]]
        lines = self._dedent(self._consume_to_next_section())
        if lines:
            before, colon, after = self._partition_field_on_colon(lines[0])
            _name, _type, _desc = '', '', lines  # type: unicode, unicode, List[unicode]

            if colon:
                if after:
                    _desc = [after] + lines[1:]
                else:
                    _desc = lines[1:]

                _type = before

            _desc = self.__class__(_desc, self._config).lines()
            return [(_name, _type, _desc,)]
        else:
            return []

    def _consume_usage_section(self):
        # type: () -> List[unicode]
        lines = self._dedent(self._consume_to_next_section())
        return lines

    def _consume_section_header(self):
        # type: () -> unicode
        section = next(self._line_iter)  # type: ignore
        stripped_section = section.strip(':')
        if stripped_section.lower() in self._sections:
            section = stripped_section
        return section

    def _consume_to_end(self):
        # type: () -> List[unicode]
        lines = []
        while self._line_iter.has_next():
            lines.append(next(self._line_iter))  # type: ignore
        return lines

    def _consume_to_next_section(self):
        # type: () -> List[unicode]
        self._consume_empty()
        lines = []
        while not self._is_section_break():
            lines.append(next(self._line_iter))  # type: ignore
        return lines + self._consume_empty()

    def _dedent(self, lines, full=False):
        # type: (List[unicode], bool) -> List[unicode]
        if full:
            return [line.lstrip() for line in lines]
        else:
            min_indent = self._get_min_indent(lines)
            return [line[min_indent:] for line in lines]

    def _escape_args_and_kwargs(self, name):
        # type: (unicode) -> unicode
        if name[:2] == '**':
            return r'\*\*' + name[2:]
        elif name[:1] == '*':
            return r'\*' + name[1:]
        else:
            return name

    def _fix_field_desc(self, desc):
        # type: (List[unicode]) -> List[unicode]
        if self._is_list(desc):
            desc = [u''] + desc
        elif desc[0].endswith('::'):
            desc_block = desc[1:]
            indent = self._get_indent(desc[0])
            block_indent = self._get_initial_indent(desc_block)
            if block_indent > indent:
                desc = [u''] + desc
            else:
                desc = ['', desc[0]] + self._indent(desc_block, 4)
        return desc

    def _format_admonition(self, admonition, lines):
        # type: (unicode, List[unicode]) -> List[unicode]
        lines = self._strip_empty(lines)
        if len(lines) == 1:
            return ['.. %s:: %s' % (admonition, lines[0].strip()), '']
        elif lines:
            lines = self._indent(self._dedent(lines), 3)
            return [u'.. %s::' % admonition, u''] + lines + [u'']
        else:
            return [u'.. %s::' % admonition, u'']

    def _format_block(self, prefix, lines, padding=None):
        # type: (unicode, List[unicode], unicode) -> List[unicode]
        if lines:
            if padding is None:
                padding = ' ' * len(prefix)
            result_lines = []
            for i, line in enumerate(lines):
                if i == 0:
                    result_lines.append((prefix + line).rstrip())
                elif line:
                    result_lines.append(padding + line)
                else:
                    result_lines.append('')
            return result_lines
        else:
            return [prefix]

    def _format_docutils_params(self, fields, field_role='param',
                                type_role='type'):
        # type: (List[Tuple[unicode, unicode, List[unicode]]], unicode, unicode) -> List[unicode]  # NOQA
        lines = []
        for _name, _type, _desc in fields:
            _desc = self._strip_empty(_desc)
            if any(_desc):
                _desc = self._fix_field_desc(_desc)
                field = ':%s %s: ' % (field_role, _name)
                lines.extend(self._format_block(field, _desc))
            else:
                lines.append(':%s %s:' % (field_role, _name))

            if _type:
                lines.append(':%s %s: %s' % (type_role, _name, _type))
        return lines + ['']

    def _format_field(self, _name, _type, _desc):
        # type: (unicode, unicode, List[unicode]) -> List[unicode]
        _desc = self._strip_empty(_desc)
        has_desc = any(_desc)
        separator = has_desc and ' -- ' or ''
        if _name:
            if _type:
                if '`' in _type:
                    field = '**%s** (%s)%s' % (_name, _type, separator)  # type: unicode
                else:
                    field = '**%s** (*%s*)%s' % (_name, _type, separator)
            else:
                field = '**%s**%s' % (_name, separator)
        elif _type:
            if '`' in _type:
                field = '%s%s' % (_type, separator)
            else:
                field = '*%s*%s' % (_type, separator)
        else:
            field = ''

        if has_desc:
            _desc = self._fix_field_desc(_desc)
            if _desc[0]:
                return [field + _desc[0]] + _desc[1:]
            else:
                return [field] + _desc
        else:
            return [field]

    def _format_fields(self, field_type, fields):
        # type: (unicode, List[Tuple[unicode, unicode, List[unicode]]]) -> List[unicode]
        field_type = ':%s:' % field_type.strip()
        padding = ' ' * len(field_type)
        multi = len(fields) > 1
        lines = []  # type: List[unicode]
        for _name, _type, _desc in fields:
            field = self._format_field(_name, _type, _desc)
            if multi:
                if lines:
                    lines.extend(self._format_block(padding + ' * ', field))
                else:
                    lines.extend(self._format_block(field_type + ' * ', field))
            else:
                lines.extend(self._format_block(field_type + ' ', field))
        if lines and lines[-1]:
            lines.append('')
        return lines

    def _get_current_indent(self, peek_ahead=0):
        # type: (int) -> int
        line = self._line_iter.peek(peek_ahead + 1)[peek_ahead]
        while line != self._line_iter.sentinel:
            if line:
                return self._get_indent(line)
            peek_ahead += 1
            line = self._line_iter.peek(peek_ahead + 1)[peek_ahead]
        return 0

    def _get_indent(self, line):
        # type: (unicode) -> int
        for i, s in enumerate(line):
            if not s.isspace():
                return i
        return len(line)

    def _get_initial_indent(self, lines):
        # type: (List[unicode]) -> int
        for line in lines:
            if line:
                return self._get_indent(line)
        return 0

    def _get_min_indent(self, lines):
        # type: (List[unicode]) -> int
        min_indent = None
        for line in lines:
            if line:
                indent = self._get_indent(line)
                if min_indent is None:
                    min_indent = indent
                elif indent < min_indent:
                    min_indent = indent
        return min_indent or 0

    def _indent(self, lines, n=4):
        # type: (List[unicode], int) -> List[unicode]
        return [(' ' * n) + line for line in lines]

    def _is_indented(self, line, indent=1):
        # type: (unicode, int) -> bool
        for i, s in enumerate(line):
            if i >= indent:
                return True
            elif not s.isspace():
                return False
        return False

    def _is_list(self, lines):
        # type: (List[unicode]) -> bool
        if not lines:
            return False
        if _bullet_list_regex.match(lines[0]):  # type: ignore
            return True
        if _enumerated_list_regex.match(lines[0]):  # type: ignore
            return True
        if len(lines) < 2 or lines[0].endswith('::'):
            return False
        indent = self._get_indent(lines[0])
        next_indent = indent
        for line in lines[1:]:
            if line:
                next_indent = self._get_indent(line)
                break
        return next_indent > indent

    def _is_section_header(self):
        # type: () -> bool
        section = self._line_iter.peek().lower()
        match = _google_section_regex.match(section)
        if match and section.strip(':') in self._sections:
            header_indent = self._get_indent(section)
            section_indent = self._get_current_indent(peek_ahead=1)
            return section_indent > header_indent
        elif self._directive_sections:
            if _directive_regex.match(section):
                for directive_section in self._directive_sections:
                    if section.startswith(directive_section):
                        return True
        return False

    def _is_section_break(self):
        # type: () -> bool
        line = self._line_iter.peek()
        return (not self._line_iter.has_next() or
                self._is_section_header() or
                (self._is_in_section and
                    line and
                    not self._is_indented(line, self._section_indent)))

    def _parse(self):
        # type: () -> None
        self._parsed_lines = self._consume_empty()

        if self._name and (self._what == 'attribute' or self._what == 'data'):
            self._parsed_lines.extend(self._parse_attribute_docstring())
            return

        while self._line_iter.has_next():
            if self._is_section_header():
                try:
                    section = self._consume_section_header()
                    self._is_in_section = True
                    self._section_indent = self._get_current_indent()
                    if _directive_regex.match(section):  # type: ignore
                        lines = [section] + self._consume_to_next_section()
                    else:
                        lines = self._sections[section.lower()](section)
                finally:
                    self._is_in_section = False
                    self._section_indent = 0
            else:
                if not self._parsed_lines:
                    lines = self._consume_contiguous() + self._consume_empty()
                else:
                    lines = self._consume_to_next_section()
            self._parsed_lines.extend(lines)

    def _parse_attribute_docstring(self):
        # type: () -> List[unicode]
        _type, _desc = self._consume_inline_attribute()
        return self._format_field('', _type, _desc)

    def _parse_attributes_section(self, section):
        # type: (unicode) -> List[unicode]
        lines = []
        for _name, _type, _desc in self._consume_fields():
            if self._config.napoleon_use_ivar:
                field = ':ivar %s: ' % _name  # type: unicode
                lines.extend(self._format_block(field, _desc))
                if _type:
                    lines.append(':vartype %s: %s' % (_name, _type))
            else:
                lines.extend(['.. attribute:: ' + _name, ''])
                fields = self._format_field('', _type, _desc)
                lines.extend(self._indent(fields, 3))
                lines.append('')
        if self._config.napoleon_use_ivar:
            lines.append('')
        return lines

    def _parse_examples_section(self, section):
        # type: (unicode) -> List[unicode]
        use_admonition = self._config.napoleon_use_admonition_for_examples
        return self._parse_generic_section(section, use_admonition)

    def _parse_usage_section(self, section):
        # type: (unicode) -> List[unicode]
        header = ['.. rubric:: Usage:', '']  # type: List[unicode]
        block = ['.. code-block:: python', '']  # type: List[unicode]
        lines = self._consume_usage_section()
        lines = self._indent(lines, 3)
        return header + block + lines + ['']

    def _parse_generic_section(self, section, use_admonition):
        # type: (unicode, bool) -> List[unicode]
        lines = self._strip_empty(self._consume_to_next_section())
        lines = self._dedent(lines)
        if use_admonition:
            header = '.. admonition:: %s' % section  # type: unicode
            lines = self._indent(lines, 3)
        else:
            header = '.. rubric:: %s' % section
        if lines:
            return [header, ''] + lines + ['']
        else:
            return [header, '']

    def _parse_keyword_arguments_section(self, section):
        # type: (unicode) -> List[unicode]
        fields = self._consume_fields()
        if self._config.napoleon_use_keyword:
            return self._format_docutils_params(
                fields,
                field_role="keyword",
                type_role="kwtype")
        else:
            return self._format_fields('Keyword Arguments', fields)

    def _parse_methods_section(self, section):
        # type: (unicode) -> List[unicode]
        lines = []  # type: List[unicode]
        for _name, _, _desc in self._consume_fields(parse_type=False):
            lines.append('.. method:: %s' % _name)
            if _desc:
                lines.extend([u''] + self._indent(_desc, 3))
            lines.append('')
        return lines

    def _parse_note_section(self, section):
        # type: (unicode) -> List[unicode]
        lines = self._consume_to_next_section()
        return self._format_admonition('note', lines)

    def _parse_notes_section(self, section):
        # type: (unicode) -> List[unicode]
        use_admonition = self._config.napoleon_use_admonition_for_notes
        return self._parse_generic_section('Notes', use_admonition)

    def _parse_other_parameters_section(self, section):
        # type: (unicode) -> List[unicode]
        return self._format_fields('Other Parameters', self._consume_fields())

    def _parse_parameters_section(self, section):
        # type: (unicode) -> List[unicode]
        fields = self._consume_fields()
        if self._config.napoleon_use_param:
            return self._format_docutils_params(fields)
        else:
            return self._format_fields('Parameters', fields)

    def _parse_raises_section(self, section):
        # type: (unicode) -> List[unicode]
        fields = self._consume_fields(parse_type=False, prefer_type=True)
        field_type = ':raises:'
        padding = ' ' * len(field_type)
        multi = len(fields) > 1
        lines = []  # type: List[unicode]
        for _, _type, _desc in fields:
            _desc = self._strip_empty(_desc)
            has_desc = any(_desc)
            separator = has_desc and ' -- ' or ''
            if _type:
                has_refs = '`' in _type or ':' in _type
                has_space = any(c in ' \t\n\v\f ' for c in _type)

                if not has_refs and not has_space:
                    _type = ':exc:`%s`%s' % (_type, separator)
                elif has_desc and has_space:
                    _type = '*%s*%s' % (_type, separator)
                else:
                    _type = '%s%s' % (_type, separator)

                if has_desc:
                    field = [_type + _desc[0]] + _desc[1:]
                else:
                    field = [_type]
            else:
                field = _desc
            if multi:
                if lines:
                    lines.extend(self._format_block(padding + ' * ', field))
                else:
                    lines.extend(self._format_block(field_type + ' * ', field))
            else:
                lines.extend(self._format_block(field_type + ' ', field))
        if lines and lines[-1]:
            lines.append('')
        return lines

    def _parse_references_section(self, section):
        # type: (unicode) -> List[unicode]
        use_admonition = self._config.napoleon_use_admonition_for_references
        return self._parse_generic_section('References', use_admonition)

    def _parse_returns_section(self, section):
        # type: (unicode) -> List[unicode]
        fields = self._consume_returns_section()
        multi = len(fields) > 1
        if multi:
            use_rtype = False
        else:
            use_rtype = self._config.napoleon_use_rtype

        lines = []  # type: List[unicode]
        for _name, _type, _desc in fields:
            if use_rtype:
                field = self._format_field(_name, '', _desc)
            else:
                field = self._format_field(_name, _type, _desc)

            if multi:
                if lines:
                    lines.extend(self._format_block('          * ', field))
                else:
                    lines.extend(self._format_block(':returns: * ', field))
            else:
                lines.extend(self._format_block(':returns: ', field))
                if _type and use_rtype:
                    lines.extend([':rtype: %s' % _type, ''])
        if lines and lines[-1]:
            lines.append('')
        return lines

    def _parse_see_also_section(self, section):
        # type: (unicode) -> List[unicode]
        lines = self._consume_to_next_section()
        return self._format_admonition('seealso', lines)

    def _parse_todo_section(self, section):
        # type: (unicode) -> List[unicode]
        lines = self._consume_to_next_section()
        return self._format_admonition('todo', lines)

    def _parse_warning_section(self, section):
        # type: (unicode) -> List[unicode]
        lines = self._consume_to_next_section()
        return self._format_admonition('warning', lines)

    def _parse_warns_section(self, section):
        # type: (unicode) -> List[unicode]
        return self._format_fields('Warns', self._consume_fields())

    def _parse_yields_section(self, section):
        # type: (unicode) -> List[unicode]
        fields = self._consume_returns_section()
        return self._format_fields('Yields', fields)

    def _partition_field_on_colon(self, line):
        # type: (unicode) -> Tuple[unicode, unicode, unicode]
        before_colon = []
        after_colon = []
        colon = ''
        found_colon = False
        for i, source in enumerate(_xref_regex.split(line)):  # type: ignore
            if found_colon:
                after_colon.append(source)
            else:
                m = _single_colon_regex.search(source)
                if (i % 2) == 0 and m:
                    found_colon = True
                    colon = source[m.start(): m.end()]
                    before_colon.append(source[:m.start()])
                    after_colon.append(source[m.end():])
                else:
                    before_colon.append(source)

        return ("".join(before_colon).strip(),
                colon,
                "".join(after_colon).strip())

    def _strip_empty(self, lines):
        # type: (List[unicode]) -> List[unicode]
        if lines:
            start = -1
            for i, line in enumerate(lines):
                if line:
                    start = i
                    break
            if start == -1:
                lines = []
            end = -1
            for i in reversed(range(len(lines))):
                line = lines[i]
                if line:
                    end = i
                    break
            if start > 0 or end + 1 < len(lines):
                lines = lines[start:end + 1]
        return lines


class NumpyDocstring(GoogleDocstring):
    """Convert NumPy style docstrings to reStructuredText.

    Parameters
    ----------
    docstring : :obj:`str` or :obj:`list` of :obj:`str`
        The docstring to parse, given either as a string or split into
        individual lines.
    config: :obj:`sphinx.ext.napoleon.Config` or :obj:`sphinx.config.Config`
        The configuration settings to use. If not given, defaults to the
        config object on `app`; or if `app` is not given defaults to the
        a new :class:`sphinx.ext.napoleon.Config` object.


    Other Parameters
    ----------------
    app : :class:`sphinx.application.Sphinx`, optional
        Application object representing the Sphinx process.
    what : :obj:`str`, optional
        A string specifying the type of the object to which the docstring
        belongs. Valid values: "module", "class", "exception", "function",
        "method", "attribute".
    name : :obj:`str`, optional
        The fully qualified name of the object.
    obj : module, class, exception, function, method, or attribute
        The object to which the docstring belongs.
    options : :class:`sphinx.ext.autodoc.Options`, optional
        The options given to the directive: an object with attributes
        inherited_members, undoc_members, show_inheritance and noindex that
        are True if the flag option of same name was given to the auto
        directive.


    Example
    -------
    >>> from sphinx.ext.napoleon import Config
    >>> config = Config(napoleon_use_param=True, napoleon_use_rtype=True)
    >>> docstring = '''One line summary.
    ...
    ... Extended description.
    ...
    ... Parameters
    ... ----------
    ... arg1 : int
    ...     Description of `arg1`
    ... arg2 : str
    ...     Description of `arg2`
    ... Returns
    ... -------
    ... str
    ...     Description of return value.
    ... '''
    >>> print(NumpyDocstring(docstring, config))
    One line summary.
    <BLANKLINE>
    Extended description.
    <BLANKLINE>
    :param arg1: Description of `arg1`
    :type arg1: int
    :param arg2: Description of `arg2`
    :type arg2: str
    <BLANKLINE>
    :returns: Description of return value.
    :rtype: str
    <BLANKLINE>

    Methods
    -------
    __str__()
        Return the parsed docstring in reStructuredText format.

        Returns
        -------
        str
            UTF-8 encoded version of the docstring.

    __unicode__()
        Return the parsed docstring in reStructuredText format.

        Returns
        -------
        unicode
            Unicode version of the docstring.

    lines()
        Return the parsed lines of the docstring in reStructuredText format.

        Returns
        -------
        list(str)
            The lines of the docstring in a list.

    """
    def __init__(self, docstring, config=None, app=None, what='', name='',
                 obj=None, options=None):
        # type: (Union[unicode, List[unicode]], SphinxConfig, Sphinx, unicode, unicode, Any, Any) -> None  # NOQA
        self._directive_sections = ['.. index::']
        super(NumpyDocstring, self).__init__(docstring, config, app, what,
                                             name, obj, options)

    def _consume_field(self, parse_type=True, prefer_type=False):
        # type: (bool, bool) -> Tuple[unicode, unicode, List[unicode]]
        line = next(self._line_iter)  # type: ignore
        if parse_type:
            _name, _, _type = self._partition_field_on_colon(line)
        else:
            _name, _type = line, ''
        _name, _type = _name.strip(), _type.strip()
        _name = self._escape_args_and_kwargs(_name)

        if prefer_type and not _type:
            _type, _name = _name, _type
        indent = self._get_indent(line) + 1
        _desc = self._dedent(self._consume_indented_block(indent))
        _desc = self.__class__(_desc, self._config).lines()
        return _name, _type, _desc

    def _consume_returns_section(self):
        # type: () -> List[Tuple[unicode, unicode, List[unicode]]]
        return self._consume_fields(prefer_type=True)

    def _consume_section_header(self):
        # type: () -> unicode
        section = next(self._line_iter)  # type: ignore
        if not _directive_regex.match(section):
            # Consume the header underline
            next(self._line_iter)  # type: ignore
        return section

    def _is_section_break(self):
        # type: () -> bool
        line1, line2 = self._line_iter.peek(2)
        return (not self._line_iter.has_next() or
                self._is_section_header() or
                ['', ''] == [line1, line2] or
                (self._is_in_section and
                    line1 and
                    not self._is_indented(line1, self._section_indent)))

    def _is_section_header(self):
        # type: () -> bool
        section, underline = self._line_iter.peek(2)
        section = section.lower()
        if section in self._sections and isinstance(underline, string_types):
            return bool(_numpy_section_regex.match(underline))  # type: ignore
        elif self._directive_sections:
            if _directive_regex.match(section):
                for directive_section in self._directive_sections:
                    if section.startswith(directive_section):
                        return True
        return False

    _name_rgx = re.compile(r"^\s*(:(?P<role>\w+):`(?P<name>[a-zA-Z0-9_.-]+)`|"
                           r" (?P<name2>[a-zA-Z0-9_.-]+))\s*", re.X)

    def _parse_see_also_section(self, section):
        # type: (unicode) -> List[unicode]
        lines = self._consume_to_next_section()
        try:
            return self._parse_numpydoc_see_also_section(lines)
        except ValueError:
            return self._format_admonition('seealso', lines)

    def _parse_numpydoc_see_also_section(self, content):
        # type: (List[unicode]) -> List[unicode]
        """
        Derived from the NumpyDoc implementation of _parse_see_also.

        See Also
        --------
        func_name : Descriptive text
            continued text
        another_func_name : Descriptive text
        func_name1, func_name2, :meth:`func_name`, func_name3

        """
        items = []

        def parse_item_name(text):
            """Match ':role:`name`' or 'name'"""
            m = self._name_rgx.match(text)
            if m:
                g = m.groups()
                if g[1] is None:
                    return g[3], None
                else:
                    return g[2], g[1]
            raise ValueError("%s is not a item name" % text)

        def push_item(name, rest):
            if not name:
                return
            name, role = parse_item_name(name)
            items.append((name, list(rest), role))
            del rest[:]

        current_func = None
        rest = []  # type: List[unicode]

        for line in content:
            if not line.strip():
                continue

            m = self._name_rgx.match(line)  # type: ignore
            if m and line[m.end():].strip().startswith(':'):
                push_item(current_func, rest)
                current_func, line = line[:m.end()], line[m.end():]
                rest = [line.split(':', 1)[1].strip()]
                if not rest[0]:
                    rest = []
            elif not line.startswith(' '):
                push_item(current_func, rest)
                current_func = None
                if ',' in line:
                    for func in line.split(','):
                        if func.strip():
                            push_item(func, [])
                elif line.strip():
                    current_func = line
            elif current_func is not None:
                rest.append(line.strip())
        push_item(current_func, rest)

        if not items:
            return []

        roles = {
            'method': 'meth',
            'meth': 'meth',
            'function': 'func',
            'func': 'func',
            'class': 'class',
            'exception': 'exc',
            'exc': 'exc',
            'object': 'obj',
            'obj': 'obj',
            'module': 'mod',
            'mod': 'mod',
            'data': 'data',
            'constant': 'const',
            'const': 'const',
            'attribute': 'attr',
            'attr': 'attr'
        }  # type: Dict[unicode, unicode]
        if self._what is None:
            func_role = 'obj'  # type: unicode
        else:
            func_role = roles.get(self._what, '')
        lines = []  # type: List[unicode]
        last_had_desc = True
        for func, desc, role in items:
            if role:
                link = ':%s:`%s`' % (role, func)
            elif func_role:
                link = ':%s:`%s`' % (func_role, func)
            else:
                link = "`%s`_" % func
            if desc or last_had_desc:
                lines += ['']
                lines += [link]
            else:
                lines[-1] += ", %s" % link
            if desc:
                lines += self._indent([' '.join(desc)])
                last_had_desc = True
            else:
                last_had_desc = False
        lines += ['']

        return self._format_admonition('seealso', lines)



import pytest
from mando import Program

from . import capture

program = Program('example.py', '1.0.10')


@program.command(doctype='google')
def simple_google_docstring(arg1, arg2="string"):
    '''One line summary.

    Extended description.

    Args:
      arg1(int): Description of `arg1`
      arg2(str): Description of `arg2`
    Returns:
      str: Description of return value.
    '''
    return int(arg1) * arg2


GENERIC_COMMAND_CASES = [
    ('simple_google_docstring 2 --arg2=test', 'testtest'),
]


@pytest.mark.parametrize('args,result', GENERIC_COMMAND_CASES)
def test_generic_command(args, result):
    args = args.split()
    assert result == program.execute(args)
    assert program.parse(args)[0].__name__ == program._current_command


GOOGLE_DOCSTRING_HELP_CASES = [
    ('simple_google_docstring --help 2 --arg2=test', '''usage: example.py simple_google_docstring [-h] [--arg2 ARG2] arg1

Extended description.

positional arguments:
  arg1         Description of `arg1`

optional arguments:
  -h, --help   show this help message and exit
  --arg2 ARG2  Description of `arg2`
'''),
]


@pytest.mark.parametrize('args,result', GOOGLE_DOCSTRING_HELP_CASES)
def test_google_docstring_help(args, result):
    args = args.split()
    with pytest.raises(SystemExit):
        with capture.capture_sys_output() as (stdout, stderr):
            program.execute(args)
    assert result == stdout.getvalue()




import argparse
import sys

from rst2ansi import rst2ansi


def b(s):
    # Useful for very coarse version differentiation.
    PY2 = sys.version_info[0] == 2
    PY3 = sys.version_info[0] == 3
    if PY3:
        return s.encode("utf-8")
    else:
        return s


class RSTHelpFormatter(argparse.RawTextHelpFormatter):
    """
    Custom formatter class that is capable of interpreting ReST.
    """
    def format_help(self):
        ret = rst2ansi(b(super(RSTHelpFormatter, self).format_help()) +
                       b('\n'))
        return ret.encode(sys.stdout.encoding,
                          'replace').decode(sys.stdout.encoding)

    def format_usage(self):
        ret = rst2ansi(b(super(RSTHelpFormatter, self).format_usage()) +
                       b('\n'))
        return ret.encode(sys.stdout.encoding,
                          'replace').decode(sys.stdout.encoding)



'''Main module containing the class Program(), which allows the conversion from
ordinary Python functions into commands for the command line. It uses
:py:module:``argparse`` behind the scenes.'''

import sys
import inspect
import argparse
try:
    getfullargspec = inspect.getfullargspec
except AttributeError:
    getfullargspec = inspect.getargspec
try:
    from itertools import izip_longest
except ImportError:  # pragma: no cover
    from itertools import zip_longest as izip_longest

from mando.napoleon import Config, GoogleDocstring, NumpyDocstring

from mando.utils import (purify_doc, action_by_type, find_param_docs,
                         split_doc, ensure_dashes, purify_kwargs)


_POSITIONAL = type('_positional', (object,), {})
_DISPATCH_TO = '_dispatch_to'


class SubProgram(object):

    def __init__(self, parser, argspecs):
        self.parser = parser
        self._subparsers = self.parser.add_subparsers()
        self._argspecs = argspecs

    @property
    def name(self):
        return self.parser.prog

    # Add global script options.
    def option(self, *args, **kwd):
        assert args and all(arg.startswith('-') for arg in args), \
            "Positional arguments not supported here"
        completer = kwd.pop('completer', None)
        arg = self.parser.add_argument(*args, **kwd)
        if completer is not None:
            arg.completer = completer
        # do not attempt to shadow existing attributes
        assert not hasattr(self, arg.dest), "Invalid option name: " + arg.dest
        return arg

    def add_subprog(self, name, **kwd):
        # also always provide help= to fix missing entry in command list
        help = kwd.pop('help', "{} subcommand".format(name))
        prog = SubProgram(self._subparsers.add_parser(name, help=help, **kwd),
                          self._argspecs)
        # do not attempt to overwrite existing attributes
        assert not hasattr(self, name), "Invalid sub-prog name: " + name
        setattr(self, name, prog)
        return prog

    def command(self, *args, **kwargs):
        '''A decorator to convert a function into a command. It can be applied
        as ``@command`` or as ``@command(new_name)``, specifying an alternative
        name for the command (default one is ``func.__name__``).'''
        if len(args) == 1 and hasattr(args[0], '__call__'):
            return self._generate_command(args[0])
        else:
            def _command(func):
                return self._generate_command(func, *args, **kwargs)
            return _command

    def arg(self, param, *args, **kwargs):
        '''A decorator to override the parameters extracted from the docstring
        or to add new ones.

        :param param: The parameter's name. It must be among the function's
            arguments names.'''
        def wrapper(func):
            if not hasattr(func, '_argopts'):
                func._argopts = {}
            func._argopts[param] = (args, kwargs)
            return func
        return wrapper

    def _generate_command(self, func, name=None, doctype='rest',
                          *args, **kwargs):
        '''Generate argparse's subparser.

        :param func: The function to analyze.
        :param name: If given, a different name for the command. The default
            one is ``func.__name__``.'''
        func_name = func.__name__
        name = func_name if name is None else name
        argspec = getfullargspec(func)
        self._argspecs[func_name] = argspec
        argz = izip_longest(reversed(argspec.args),
                            reversed(argspec.defaults or []),
                            fillvalue=_POSITIONAL())
        argz = reversed(list(argz))
        doc = (inspect.getdoc(func) or '').strip() + '\n'
        if doctype == 'numpy':
            config = Config(napoleon_google_docstring=False,
                            napoleon_use_rtype=False)
            doc = str(NumpyDocstring(doc, config))
        elif doctype == 'google':
            config = Config(napoleon_numpy_docstring=False,
                            napoleon_use_rtype=False)
            doc = str(GoogleDocstring(doc, config))
        elif doctype == 'rest':
            pass
        else:
            raise ValueError('doctype must be one of "numpy", "google", '
                             'or "rest"')
        cmd_help, cmd_desc = split_doc(purify_doc(doc))
        subparser = self._subparsers.add_parser(name,
                                                help=cmd_help or None,
                                                description=cmd_desc or None,
                                                **kwargs)
        params = find_param_docs(doc)
        for a, kw in self._analyze_func(func, params, argz, argspec.varargs):
            completer = kw.pop('completer', None)
            arg = subparser.add_argument(*a, **purify_kwargs(kw))
            if completer is not None:
                arg.completer = completer

        subparser.set_defaults(**{_DISPATCH_TO: func})
        return func

    def _analyze_func(self, func, params, argz, varargs_name):
        '''Analyze the given function, merging default arguments, overridden
        arguments (with @arg) and parameters extracted from the docstring.

        :param func: The function to analyze.
        :param params: Parameters extracted from docstring.
        :param argz: A list of the form (arg, default), containing arguments
            and their default value.
        :param varargs_name: The name of the variable arguments, if present,
            otherwise ``None``.'''
        for arg, default in argz:
            override = getattr(func, '_argopts', {}).get(arg, ((), {}))
            yield merge(arg, default, override, *params.get(arg, ([], {})))
        if varargs_name is not None:
            kwargs = {'nargs': '*'}
            kwargs.update(params.get(varargs_name, (None, {}))[1])
            yield ([varargs_name], kwargs)


class Program(SubProgram):

    def __init__(self, prog=None, version=None, **kwargs):
        parser = argparse.ArgumentParser(prog, **kwargs)
        if version is not None:
            parser.add_argument('-v', '--version', action='version',
                                version=version)

        super(Program, self).__init__(parser, dict())
        self._options = None
        self._current_command = None

    # Attribute lookup fallback redirecting to (internal) options instance.
    def __getattr__(self, attr):
        return getattr(self._options, attr)

    def parse(self, args):
        '''Parse the given arguments and return a tuple ``(command, args)``,
        where ``args`` is a list consisting of all arguments. The command can
        then be called as ``command(*args)``.

        :param args: The arguments to parse.'''
        try:
            # run completion handler before parsing
            import argcomplete
            argcomplete.autocomplete(self.parser)
        except ImportError:  # pragma: no cover
            # ignore error if not installed
            pass

        self._options = self.parser.parse_args(args)
        arg_map = self._options.__dict__
        if _DISPATCH_TO not in arg_map:  # pragma: no cover
            self.parser.error("too few arguments")

        command = arg_map.pop(_DISPATCH_TO)
        argspec = self._argspecs[command.__name__]
        real_args = []
        for arg in argspec.args:
            real_args.append(arg_map.pop(arg))
        if arg_map and arg_map.get(argspec.varargs):
            real_args.extend(arg_map.pop(argspec.varargs))
        return command, real_args

    def execute(self, args):
        '''Parse the arguments and execute the resulting command.

        :param args: The arguments to parse.'''
        command, a = self.parse(args)
        self._current_command = command.__name__
        return command(*a)

    def __call__(self):  # pragma: no cover
        '''Parse ``sys.argv`` and execute the resulting command.'''
        return self.execute(sys.argv[1:])


def merge(arg, default, override, args, kwargs):
    '''Merge all the possible arguments into a tuple and a dictionary.

    :param arg: The argument's name.
    :param default: The argument's default value or an instance of _POSITIONAL.
    :param override: A tuple containing (args, kwargs) given to @arg.
    :param args: The arguments extracted from the docstring.
    :param kwargs: The keyword arguments extracted from the docstring.'''
    opts = [arg]
    if not isinstance(default, _POSITIONAL):
        opts = list(ensure_dashes(args or opts))
        kwargs.update({'default': default, 'dest': arg})
        kwargs.update(action_by_type(default))
    else:
        # positionals can't have a metavar, otherwise the help is screwed
        # if one really wants the metavar, it can be added with @arg
        kwargs['metavar'] = None
    kwargs.update(override[1])
    return override[0] or opts, kwargs



# This is important: it will make all literals unicode under 2.x
from __future__ import unicode_literals

import unittest

from mando import Program


program = Program('example.py', '1.0.10')


class Test_unicode_docstring_on_py2(unittest.TestCase):

    def test_py2_unicode_literals(self):
        @program.command
        def some_command():
            'this is a unicode doc-string!'

        assert not isinstance(some_command.__doc__, bytes)
        # TODO: check that the generated help is correct



import re
import textwrap

SPHINX_RE = re.compile(
    r'^([\t ]*):'
    r'(?P<field>param|type|returns|rtype|parameter|arg|argument|key|keyword)'
    r' ?(?P<var1>[-\w_]+,?)?'
    r' ?(?P<var2>[-<>\w_]+)?'
    r' ?(?P<var3>[<>\w_]+)?:'
    r'(?P<help>[^\n]*\n+((\1[ \t]+[^\n]*\n)|\n)*)',
    re.MULTILINE)
ARG_RE = re.compile(
    r'-(?P<long>-)?'
    r'(?P<key>(?(long)[^ =,]+|.))[ =]?'
    r'(?P<meta>[^ ,]+)?')
POS_RE = re.compile(
    r'(?P<meta>[^ ,]+)?')
ARG_TYPE_MAP = {
    'n': int, 'num': int, 'number': int,
    'i': int, 'int': int, 'integer': int,
    's': str, 'str': str, 'string': str,
    'f': float, 'float': float,
    None: None, '': None,
}


def purify_doc(string):
    '''Remove Sphinx's :param: and :type: lines from the docstring.'''
    return SPHINX_RE.sub('', string).rstrip()


def split_doc(string):
    '''Split the documentation into help and description.

    A two-value list is returned, of the form ``[help, desc]``. If no
    description is provided, the help is duplicated.'''
    parts = [part.strip() for part in string.split('\n\n', 1)]
    if len(parts) == 1:
        return parts * 2
    return parts


def purify_kwargs(kwargs):
    '''If type or metavar are set to None, they are removed from kwargs.'''
    for key, value in kwargs.copy().items():
        if key in set(['type', 'metavar']) and value is None:
            del kwargs[key]
    return kwargs


def find_param_docs(docstring):
    '''Find Sphinx's :param:, :type:, :returns:, and :rtype: lines and return
       a dictionary of the form:
       ``param: (opts, {metavar: meta, type: type, help: help})``.'''
    paramdocs = {}
    typedocs = {}
    for m in SPHINX_RE.finditer(docstring + '\n'):
        if m.group('field') in ['param',
                                'parameter',
                                'arg',
                                'argument',
                                'key',
                                'keyword']:
            # mando
            #     :param name: Help text.               name   None   None    0
            #     :param name <type>: Help text.        name   <type> None    1
            #     :param -n: Help text.                 -n     None   None    2
            #     :param -n <type>: Help text.          -n     <type> None    3
            #     :param --name: Help text.             --name None   None    4
            #     :param --name <type>: Help text.      --name <type> None    5
            #     :param -n, --name: Help text.         -n,    --name None    6
            #     :param -n, --name <type>: Help text.  -n,    --name <type>  7
            # sphinx
            #     :param name: Help text.               name   None   None    8
            #     :param type name: Help text.          type   name   None    9
            #     :type name: str

            # The following is ugly, but it allows for backward compatibility

            if m.group('var2') is None:  # 0, 2, 4, 8
                vname = m.group('var1')
                vtype = None
            # 1, 3, 5
            elif m.group('var2') is not None and '<' in m.group('var2'):
                vname = m.group('var1')
                vtype = m.group('var2')
            elif '-' in m.group('var1') and '-' in m.group('var2'):  # 6, 7
                vname = '{0} {1}'.format(m.group('var1'), m.group('var2'))
                vtype = m.group('var3')
            else:                        # 9
                vname = m.group('var2')
                vtype = m.group('var1')

            name, opts, meta = get_opts('{0} {1}'.format(vname.strip(),
                                                         vtype or ''))
            name = name.replace('-', '_')

            helpdoc = m.group('help').strip()
            helpdoc = helpdoc.splitlines(True)
            if len(helpdoc) > 1:
                helpdoc = helpdoc[0] + textwrap.dedent(''.join(helpdoc[1:]))
            else:
                helpdoc = helpdoc[0]
            paramdocs[name] = (opts, {
                'metavar': meta or None,
                'type': ARG_TYPE_MAP.get(meta.strip('<>')),
                'help': helpdoc,
            })
        elif m.group('field') == 'type':
            typedocs[m.group('var1').strip()] = m.group('help').strip()
    for key in typedocs:
        paramdocs[key][1]['type'] = ARG_TYPE_MAP.get(typedocs[key])
    return paramdocs


def get_opts(param):
    '''Extract options from a parameter name.'''
    if param.startswith('-'):
        opts = []
        names = []
        meta = None
        for long, name, meta in ARG_RE.findall(param):
            prefix = ['-', '--'][len(long)]
            opts.append('{0}{1}'.format(prefix, name))
            names.append(name)
        return max(names, key=len), opts, meta
    opt, meta = (list(filter(None, POS_RE.findall(param))) + [''])[:2]
    return opt, [opt], meta


def action_by_type(obj):
    '''Determine an action and a type for the given object if possible.'''
    kw = {}
    if isinstance(obj, bool):
        return {'action': ['store_true', 'store_false'][obj]}
    elif isinstance(obj, list):
        kw = {'action': 'append'}
    kw.update(get_type(obj))
    return kw


def get_type(obj):
    '''Determine the type of the object if among some of the built-in ones.'''
    otype = type(obj)
    if any(otype is t for t in set([int, float, str, bool])):
        return {'type': otype}
    return {}


def ensure_dashes(opts):
    '''Ensure that the options have the right number of dashes.'''
    for opt in opts:
        if opt.startswith('-'):
            yield opt
        else:
            yield '-' * (1 + 1 * (len(opt) > 1)) + opt



#!/usr/bin/env python

'''
Capture function
----------------------------------

'''

import sys
from contextlib import contextmanager

try:
    from cStringIO import StringIO
except ImportError:
    from io import StringIO


@contextmanager
def capture_sys_output():
    capture_out, capture_err = StringIO(), StringIO()
    current_out, current_err = sys.stdout, sys.stderr
    try:
        sys.stdout, sys.stderr = capture_out, capture_err
        yield capture_out, capture_err
    finally:
        sys.stdout, sys.stderr = current_out, current_err



import pytest
from mando.utils import action_by_type, ensure_dashes, find_param_docs, split_doc


ACTION_BY_TYPE_CASES = [
    (True, {'action': 'store_false'}),
    (False, {'action': 'store_true'}),
    ([], {'action': 'append'}),
    ([1, False], {'action': 'append'}),
    (None, {}),
    (1, {'type': type(1)}),
    (1.1, {'type': type(1.1)}),
    ('1', {'type': type('1')}),
]


@pytest.mark.parametrize('obj,result', ACTION_BY_TYPE_CASES)
def test_action_by_type(obj, result):
        assert result == action_by_type(obj)


ENSURE_DASHES_CASES = [
    (['m'], ['-m']),
    (['m', 'min'], ['-m', '--min']),
    (['-m'], ['-m']),
    (['-m', 'min'], ['-m', '--min']),
    (['m', '--min'], ['-m', '--min']),
    (['-m', '--min'], ['-m', '--min']),
    (['-m', '--min', 'l', 'less'], ['-m', '--min', '-l', '--less']),
]


@pytest.mark.parametrize('opts,result', ENSURE_DASHES_CASES)
def test_ensure_dashes(opts, result):
    assert result == list(ensure_dashes(opts))


SPLIT_DOC_CASES = [
    ('', ['', '']),
    ('only help.', ['only help.', 'only help.']),
    ('help.\nstill help.', ['help.\nstill help.', 'help.\nstill help.']),
    ('help\n\ndesc', ['help', 'desc']),
    ('help\n\n\ndesc\n', ['help', 'desc']),
]


@pytest.mark.parametrize('doc,parts', SPLIT_DOC_CASES)
def test_split_doc(doc, parts):
    assert parts == split_doc(doc)


a_1 = {'a_param': (['a-param'], {'help': 'Short story.'})}
a_1_1 = {'a_param': (['a_param'], {'help': 'Short story.'})}
a_2 = {'j': (['-j'], {'help': 'Woow'})}
a_3 = {'noun': (['-n', '--noun'], {'help': 'cat'})}
a_all = {}
for a in (a_1, a_2, a_3):
    a_all.update(a)


FIND_PARAM_CASES = [
    ('', {}),
    ('Brevity is the soul of wit.', {}),
    (':param a-param: Short story.', a_1),
    (':param a_param: Short story.', a_1_1),
    (':param -j: Woow', a_2),
    (':param -n, --noun: cat', a_3),
    ('''
         Some short text here and there.

         :param well: water''', {'well': (['well'], {'help': 'water'})}),
    ('''
         :param a-param: Short story.
         :param -j: Woow
         :param -n, --noun: cat''', a_all),
    ('''
         Lemme see.

         :param long-story: A long story believe me: when all started, Adam and Bob were just two little farmers.
         ''', {'long_story': (['long-story'], {'help': 'A long story '
                              'believe me: when all started, Adam and '
                              'Bob were just two little farmers.'})}),
]


@pytest.mark.parametrize('doc,params', FIND_PARAM_CASES)
def test_find_param(doc, params):
        found_params = find_param_docs(doc)
        assert params.keys() == found_params.keys()
        for key, value in params.items():
            assert key in found_params
            found_value = found_params[key]
            assert value[0] == found_value[0]
            for kwarg, val in value[1].items():
                assert val == found_value[1][kwarg]



# -*- coding: utf-8 -*-
"""
    sphinx.ext.napoleon
    ~~~~~~~~~~~~~~~~~~~

    Support for NumPy and Google style docstrings.

    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""

from six import iteritems

from mando.napoleon.docstring import GoogleDocstring, NumpyDocstring


class Config(object):
    """Sphinx napoleon extension settings in `conf.py`.

    Listed below are all the settings used by napoleon and their default
    values. These settings can be changed in the Sphinx `conf.py` file. Make
    sure that both "sphinx.ext.autodoc" and "sphinx.ext.napoleon" are
    enabled in `conf.py`::

        # conf.py

        # Add any Sphinx extension module names here, as strings
        extensions = ['sphinx.ext.autodoc', 'sphinx.ext.napoleon']

        # Napoleon settings
        napoleon_google_docstring = True
        napoleon_numpy_docstring = True
        napoleon_include_init_with_doc = False
        napoleon_include_private_with_doc = False
        napoleon_include_special_with_doc = False
        napoleon_use_admonition_for_examples = False
        napoleon_use_admonition_for_notes = False
        napoleon_use_admonition_for_references = False
        napoleon_use_ivar = False
        napoleon_use_param = True
        napoleon_use_rtype = True
        napoleon_use_keyword = True

    .. _Google style:
       http://google.github.io/styleguide/pyguide.html
    .. _NumPy style:
       https://github.com/numpy/numpy/blob/master/doc/HOWTO_DOCUMENT.rst.txt

    Attributes
    ----------
    napoleon_google_docstring : :obj:`bool` (Defaults to True)
        True to parse `Google style`_ docstrings. False to disable support
        for Google style docstrings.
    napoleon_numpy_docstring : :obj:`bool` (Defaults to True)
        True to parse `NumPy style`_ docstrings. False to disable support
        for NumPy style docstrings.
    napoleon_include_init_with_doc : :obj:`bool` (Defaults to False)
        True to list ``__init___`` docstrings separately from the class
        docstring. False to fall back to Sphinx's default behavior, which
        considers the ``__init___`` docstring as part of the class
        documentation.

        **If True**::

            def __init__(self):
                \"\"\"
                This will be included in the docs because it has a docstring
                \"\"\"

            def __init__(self):
                # This will NOT be included in the docs

    napoleon_include_private_with_doc : :obj:`bool` (Defaults to False)
        True to include private members (like ``_membername``) with docstrings
        in the documentation. False to fall back to Sphinx's default behavior.

        **If True**::

            def _included(self):
                \"\"\"
                This will be included in the docs because it has a docstring
                \"\"\"
                pass

            def _skipped(self):
                # This will NOT be included in the docs
                pass

    napoleon_include_special_with_doc : :obj:`bool` (Defaults to False)
        True to include special members (like ``__membername__``) with
        docstrings in the documentation. False to fall back to Sphinx's
        default behavior.

        **If True**::

            def __str__(self):
                \"\"\"
                This will be included in the docs because it has a docstring
                \"\"\"
                return unicode(self).encode('utf-8')

            def __unicode__(self):
                # This will NOT be included in the docs
                return unicode(self.__class__.__name__)

    napoleon_use_admonition_for_examples : :obj:`bool` (Defaults to False)
        True to use the ``.. admonition::`` directive for the **Example** and
        **Examples** sections. False to use the ``.. rubric::`` directive
        instead. One may look better than the other depending on what HTML
        theme is used.

        This `NumPy style`_ snippet will be converted as follows::

            Example
            -------
            This is just a quick example

        **If True**::

            .. admonition:: Example

               This is just a quick example

        **If False**::

            .. rubric:: Example

            This is just a quick example

    napoleon_use_admonition_for_notes : :obj:`bool` (Defaults to False)
        True to use the ``.. admonition::`` directive for **Notes** sections.
        False to use the ``.. rubric::`` directive instead.

        Note
        ----
        The singular **Note** section will always be converted to a
        ``.. note::`` directive.

        See Also
        --------
        :attr:`napoleon_use_admonition_for_examples`

    napoleon_use_admonition_for_references : :obj:`bool` (Defaults to False)
        True to use the ``.. admonition::`` directive for **References**
        sections. False to use the ``.. rubric::`` directive instead.

        See Also
        --------
        :attr:`napoleon_use_admonition_for_examples`

    napoleon_use_ivar : :obj:`bool` (Defaults to False)
        True to use the ``:ivar:`` role for instance variables. False to use
        the ``.. attribute::`` directive instead.

        This `NumPy style`_ snippet will be converted as follows::

            Attributes
            ----------
            attr1 : int
                Description of `attr1`

        **If True**::

            :ivar attr1: Description of `attr1`
            :vartype attr1: int

        **If False**::

            .. attribute:: attr1

               *int*

               Description of `attr1`

    napoleon_use_param : :obj:`bool` (Defaults to True)
        True to use a ``:param:`` role for each function parameter. False to
        use a single ``:parameters:`` role for all the parameters.

        This `NumPy style`_ snippet will be converted as follows::

            Parameters
            ----------
            arg1 : str
                Description of `arg1`
            arg2 : int, optional
                Description of `arg2`, defaults to 0

        **If True**::

            :param arg1: Description of `arg1`
            :type arg1: str
            :param arg2: Description of `arg2`, defaults to 0
            :type arg2: int, optional

        **If False**::

            :parameters: * **arg1** (*str*) --
                           Description of `arg1`
                         * **arg2** (*int, optional*) --
                           Description of `arg2`, defaults to 0

    napoleon_use_keyword : :obj:`bool` (Defaults to True)
        True to use a ``:keyword:`` role for each function keyword argument.
        False to use a single ``:keyword arguments:`` role for all the
        keywords.

        This behaves similarly to  :attr:`napoleon_use_param`. Note unlike
        docutils, ``:keyword:`` and ``:param:`` will not be treated the same
        way - there will be a separate "Keyword Arguments" section, rendered
        in the same fashion as "Parameters" section (type links created if
        possible)

        See Also
        --------
        :attr:`napoleon_use_param`

    napoleon_use_rtype : :obj:`bool` (Defaults to True)
        True to use the ``:rtype:`` role for the return type. False to output
        the return type inline with the description.

        This `NumPy style`_ snippet will be converted as follows::

            Returns
            -------
            bool
                True if successful, False otherwise

        **If True**::

            :returns: True if successful, False otherwise
            :rtype: bool

        **If False**::

            :returns: *bool* -- True if successful, False otherwise

    """
    _config_values = {
        'napoleon_google_docstring': (True, 'env'),
        'napoleon_numpy_docstring': (True, 'env'),
        'napoleon_include_init_with_doc': (False, 'env'),
        'napoleon_include_private_with_doc': (False, 'env'),
        'napoleon_include_special_with_doc': (False, 'env'),
        'napoleon_use_admonition_for_examples': (False, 'env'),
        'napoleon_use_admonition_for_notes': (False, 'env'),
        'napoleon_use_admonition_for_references': (False, 'env'),
        'napoleon_use_ivar': (False, 'env'),
        'napoleon_use_param': (True, 'env'),
        'napoleon_use_rtype': (True, 'env'),
        'napoleon_use_keyword': (True, 'env')
    }

    def __init__(self, **settings):
        # type: (Any) -> None
        for name, (default, rebuild) in iteritems(self._config_values):
            setattr(self, name, default)
        for name, value in iteritems(settings):
            setattr(self, name, value)



from six import PY3


# UnicodeMixin
if PY3:
    class UnicodeMixin(object):
        """Mixin class to handle defining the proper __str__/__unicode__
        methods in Python 2 or 3."""

        def __str__(self):
            return self.__unicode__()
else:
    class UnicodeMixin(object):
        """Mixin class to handle defining the proper __str__/__unicode__
        methods in Python 2 or 3."""

        def __str__(self):
            return self.__unicode__().encode('utf8')



import pytest
from mando import Program


program = Program('example.py', '1.0.10')


def NoopCompleter(prefix, **kwd):
    return []


program.option(
    "-f", "--foo", dest='foo', default='bar', completer=NoopCompleter,
    help="Real programmers don't comment their code. \
          If it was hard to write, it should be hard to read."
)

program.add_subprog('sub')
program.sub.option(
    "-i", "--inc", dest='inc', type=int, default=0,
    help="Some help text."
)


@program.command
def getopt(name):
    '''
        :param name: Name of option to return.
    '''
    # also allows for: Script.foo
    return getattr(program, name)


@program.sub.command
def powOfSub(b, e):
    '''
        :param b: Base.
        :param e: Exponent.
    '''
    return int(b) ** int(e) + program.inc


@program.sub.command('powOfSub2')
def powOfSub2_impl(b, e):
    '''
        :param b: Base.
        :param e: Exponent.
    '''
    return int(b) ** int(e) - program.inc


@program.command
def goo(pos, verbose=False, bar=None):
    pass


@program.command
def vara(pos, foo, spam=24, *vars):
    '''
    :param vars: Yeah, you got it right, the variable arguments.
    '''
    pass


@program.command
def another(baw, owl=42, json=False, tomawk=None):
    '''This yet another example showcasing the power of Mando!

    :param baw: That's the positional argument, obviously.
    :param -o, --owl: Yeah, I know, this is too much.
    :param -j, --json: In case you want to pipe it through something.
    :param -t, --tomawk: Well, in this case -t isn't for time.'''
    pass


@program.command('alias')
def analiased(a, b=4):
    pass


@program.command
def power(x, y=2):
    return int(x) ** y


@program.command('more-power')
def more_power(x, y=2):
    '''This one really shows off complete power.

    :param x <int>: Well, the base.
    :param -y <int>: You got it, the exponent.'''

    return x ** y


@program.command('more-powerful')
@program.arg('x', type=int, completer=NoopCompleter)
@program.arg('y', '-y', '--epsilon', type=int)
def more_power_2(x, y=2):
    return x ** y


@program.command
@program.arg('x', type=int)
@program.arg('y', type=int)
def overriding(x, y=4):
    '''Yoo an override test.

    :param x <str>: This is so wroong!!! Let's hope it gets overridden by @arg.
    :param -y <metavar>: This too!!'''

    return x - y


@program.command
def dashes(a, b=5):
    '''Usual command help.

    :param a <int>: A help obviously.
    :param b <int>: Yooo.'''
    return a ** b


@program.command
def append(acc=[]):
    return acc


GENERIC_COMMANDS_CASES = [
    ('goo 2', [['2', False, None]]),
    ('goo 2 --verbose', [['2', True, None]]),
    ('goo 2 --bar 9', [['2', False, '9']]),
    ('goo 2 --verbose --bar 8', [['2', True, '8']]),
    ('vara 2 3', [['2', '3', 24]]),
    ('vara 2 3 --spam 8', [['2', '3', 8]]),
    # Unfortunately this is an argparse "bug". See:
    # http://bugs.python.org/issue15112
    # You cannot intermix positional and optional arguments for now.
    #('vara 1 2 --spam 8 9 8', ['1', '2', 8, '9', '8']),
    ('vara 1 2 4 5 --spam 8', [['1', '2', 8, '4', '5']]),
    ('vara --spam 8 1 2 4 5', [['1', '2', 8, '4', '5']]),
    ('vara 9 8 1 2 3 4', [['9', '8', 24, '1', '2', '3', '4']]),
    ('another 2', [['2', 42, False, None]]),
    ('another 2 -j', [['2', 42, True, None]]),
    ('another 2 -t 1 -o 3', [['2', 3, False, '1']]),
    ('another 2 --owl 89 --tomawk 98', [['2', 89, False, '98']]),
    ('another 2 --json -o 1', [['2', 1, True, None]]),
    ('another 3 --owl 8 --json --tomawk 8', [['3', 8, True, '8']]),
    ('alias 5 -b 9', [['5', 9], 'analiased']),
    ('more-power 9 -y 2', [[9, 2], 'more_power']),
    ('more-powerful 9 -y 3', [[9, 3], 'more_power_2']),
    ('more-powerful 9 --epsilon 3', [[9, 3], 'more_power_2']),
    ('overriding 2', [[2, 4]]),
    ('overriding 2 -y 7', [[2, 7]]),
    ('dashes 2', [[2, 5]]),
    ('dashes 8 -b 7', [[8, 7]]),
    ('append', [[[]]]),
    ('append --acc 2', [[['2']]]),
    ('append --acc 2 --acc 3', [[['2', '3']]]),
]


@pytest.mark.parametrize('args,rest', GENERIC_COMMANDS_CASES)
def test_generic_commands(args, rest):
    args = args.split()
    if len(rest) == 1:
        to_args = rest[0]
        real_name = args[0]
    else:
        to_args = rest[0]
        real_name = rest[1]
    parsed = program.parse(args)
    assert real_name == parsed[0].__name__
    assert to_args == parsed[1]


PROGRAM_EXECUTE_CASES = [
    ('power 2', 4),
    ('power 2 -y 4', 16),
    ('more-power 3', 9),
    ('more-power 3 -y 4', 81),
    ('more-powerful 4 -y 2', 16),
    ('more-powerful 4 --epsilon 2', 16),
    ('overriding 2', -2),
    ('overriding 2 -y 7', -5),
    ('dashes 2', 32),
    ('dashes 7 -b 3', 343),
]


@pytest.mark.parametrize('args,result', PROGRAM_EXECUTE_CASES)
def test_program_execute(args, result):
    args = args.split()
    assert result == program.execute(args)
    assert program.parse(args)[0].__name__ == program._current_command


PROGRAM_OPTIONS_CASES = [
    ('          getopt foo', 'bar'),
    ('   -f xyz getopt foo', 'xyz'),
    ('--foo xyz getopt foo', 'xyz'),
    ('          sub         powOfSub  2 3', 8),
    ('   -f xyz sub    -i 1 powOfSub  2 3', 9),
    ('--foo xyz sub --inc 2 powOfSub  2 3', 10),
    ('          sub         powOfSub2 2 3', 8),
    ('   -f xyz sub    -i 1 powOfSub2 2 3', 7),
    ('--foo xyz sub --inc 2 powOfSub2 2 3', 6),
]


@pytest.mark.parametrize('args,result', PROGRAM_OPTIONS_CASES)
def test_program_options(args, result):
    args = args.split()
    assert "example.py" == program.name
    assert result == program.execute(args)

